' FindTrustedAndSPNAccounts.vbs
' Copyright 2014 Ben Lincoln
' http://www.beneaththewaves.net/
' 

' FindTrustedAndSPNAccounts.vbs is free software: you can redistribute it and/or modify
' it under the terms of version 3 of the GNU General Public License as published by
' the Free Software Foundation.

' FindTrustedAndSPNAccounts.vbs is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with FindTrustedAndSPNAccounts.vbs (in the file LICENSE.txt).  
' If not, see <http://www.gnu.org/licenses/>.

Option Explicit

Dim objRootDSE
Dim strDNSDomain
Dim objCommand
Dim objConnection
Dim strBase
Dim strFilter
Dim objREx
Dim strAttributes
Dim strQuery
Dim objRecordSet
Dim strDN
Dim strSAMAccountName
Dim strUserPrincipalName
Dim strDescription
Dim objDescriptionElement
Dim strDescriptionPrev
Dim strAllowedToLogonTo
Dim objAllowedToLogonToElement
Dim strAllowedToLogonToPrev
Dim strSPNs
Dim objSPNsElement
Dim strSPNsPrev
Dim strAllowedToDelegateTo
Dim objAllowedToDelegateToElement
Dim strAllowedToDelegateToPrev
Dim strDisplayName
Dim intUserAccountControl
Dim intIsTrustedAccount
Dim intIncludeInResults
Dim intTrustedForDelegation
Dim intTrustedToAuthForDelegation
Dim strTrustLevel
Dim strOutLine
Dim strObjectClass
Dim strObjectClassPrev
Dim objObjectClassElement

' Determine DNS domain name.
Set objRootDSE = GetObject("LDAP://RootDSE")
strDNSDomain = objRootDSE.Get("defaultNamingContext")

' Use ADO to search Active Directory.
Set objCommand = CreateObject("ADODB.Command")
Set objConnection = CreateObject("ADODB.Connection")
objConnection.Provider = "ADsDSOObject"
objConnection.Open "Active Directory Provider"
objCommand.ActiveConnection = objConnection
strBase = "<LDAP://" & strDNSDomain & ">"

strFilter = "(|(objectClass=user)(objectClass=person))"

strAttributes = _
	"sAMAccountName,userPrincipalName,distinguishedName,displayName,userAccountControl,userWorkstations,servicePrincipalName,msDS-AllowedToDelegateTo,objectClass"
strQuery = strBase & ";" & strFilter & ";" & strAttributes & ";subtree"
objCommand.CommandText = strQuery
objCommand.Properties("Page Size") = 1000000
objCommand.Properties("Timeout") = 300
objCommand.Properties("Cache Results") = False
Set objRecordSet = objCommand.Execute

If objRecordSet.EOF Then
	Wscript.Echo "Error: no results found."
	Wscript.Quit
End If

WScript.Echo "SAM_Account_Name" & vbTab & "User_Principal_Name" & vbTab & "Distinguished_Name" & vbTab & "Display_Name" & vbTab & "Object_Class" & vbTab & "Description" & vbTab & "Allowed_To_Logon_To" & vbTab & "Service_Principal_Names" & vbTab & "Trust_Level" & vbTab & "Allowed_To_Delegate_To"

Do Until objRecordSet.EOF
	strDN = objRecordSet.Fields("distinguishedName")

	strSAMAccountName = Trim(objRecordSet.Fields("sAMAccountName"))
	If strSAMAccountName & "" = "" Then
		strSAMAccountName = "[ Unknown ]"
	End If
	
	strUserPrincipalName = Trim(objRecordSet.Fields("userPrincipalName"))
	If strUserPrincipalName & "" = "" Then
		strUserPrincipalName = "[ Unknown ]"
	End If

	On Error Resume Next
	strDescription = ""
	For Each objDescriptionElement In objRecordSet.Fields("description").Value
		strDescriptionPrev = strDescription
		strDescription = strDescription & " " & objDescriptionElement
		If Err.Number <> 0 Then
			strDescription = strDescriptionPrev
			Err.Clear
		End If
	Next
	On Error GoTo 0
	
	intIsTrustedAccount = 0
		
	On Error Resume Next
	strAllowedToDelegateTo = ""
	For Each objAllowedToDelegateToElement In objRecordSet.Fields("msDS-AllowedToDelegateTo").Value
		strAllowedToDelegateToPrev = strAllowedToDelegateTo
		strAllowedToDelegateTo = strAllowedToDelegateTo & "," & objAllowedToDelegateToElement
		If Err.Number <> 0 Then
			strAllowedToDelegateTo = strAllowedToDelegateToPrev
			Err.Clear
		End If
	Next
	On Error GoTo 0
	
	On Error Resume Next
	strObjectClass = ""
	For Each objObjectClassElement In objRecordSet.Fields("objectClass").Value
		strObjectClassPrev = strObjectClass
		strObjectClass = strObjectClass & "," & objObjectClassElement
		If Err.Number <> 0 Then
			strObjectClass = strObjectClassPrev
			Err.Clear
		End If
	Next
	On Error GoTo 0
	
	if Trim(strAllowedToDelegateTo) <> "" Then
		intIsTrustedAccount = 1
	End If
	
	On Error Resume Next
	strAllowedToLogonTo = ""
	For Each objAllowedToLogonToElement In objRecordSet.Fields("userWorkstations").Value
		strAllowedToLogonToPrev = strAllowedToLogonTo
		strAllowedToLogonTo = strAllowedToLogonTo & "," & objAllowedToLogonToElement
		If Err.Number <> 0 Then
			strAllowedToLogonTo = strAllowedToLogonToPrev
			Err.Clear
		End If
	Next
	On Error GoTo 0	
	
	If strAllowedToLogonTo = "" Then
		strAllowedToLogonTo = "[ Any System ]"
	End If
	
	On Error Resume Next
	strSPNs = ""
	For Each objSPNsElement In objRecordSet.Fields("servicePrincipalName").Value
		strSPNsPrev = strSPNs
		strSPNs = strSPNs & "," & objSPNsElement
		If Err.Number <> 0 Then
			strSPNs = strSPNsPrev
			Err.Clear
		End If
	Next
	On Error GoTo 0	
	
	intIncludeInResults = 0
	
	If Trim(strSPNs) <> "" Then
		intIncludeInResults = 1
	End If
	
	strDisplayName = Trim(objRecordSet.Fields("displayName"))

	On Error Resume Next
	intUserAccountControl = objRecordSet.Fields("userAccountControl").Value
	If Err.Number <> 0 Then
		intUserAccountControl = 0
		Err.Clear
	End If
	On Error GoTo 0
	
	intTrustedForDelegation = 0
	intTrustedToAuthForDelegation = 0
	If (intUserAccountControl AND 524288) = 524288 Then
		intTrustedForDelegation = 1
		intIsTrustedAccount = 1
	End If
	If (intUserAccountControl AND 16777216) = 16777216 Then
		intTrustedToAuthForDelegation = 1
		intIsTrustedAccount = 1
	End If

	strTrustLevel = "None"
	If intIsTrustedAccount > 0 Then
		intIncludeInResults = 1
		strTrustLevel = "Constrained"
		If Trim(strAllowedToDelegateTo) = "" Then
			strTrustLevel = "Unconstrained"
		End If
	End If
	
	If intIncludeInResults > 0 Then
		strOutLine = strSAMAccountName & vbTab & strUserPrincipalName & vbTab & strDN & vbTab & strDisplayName & vbTab & strObjectClass & vbTab & strDescription & vbTab & strAllowedToLogonTo & vbTab & strSPNs & vbTab & strTrustLevel & vbTab & strAllowedToDelegateTo

		WScript.Echo strOutLine
	End If


  objRecordSet.MoveNext
Loop
